@@ -1,4 +1,4 @@
-# Copyright (c) 2016 RIPE NCC
+# Copyright (c) 2023 RIPE NCC
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,12 +13,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Optional
 from datetime import datetime
 import argparse
 import importlib
 import os
 import pkgutil
-import re
 import sys
 import platform
 
@@ -45,7 +45,7 @@ def _get_command_name(cmd_or_factory):
 class Command(object):
     # If NAME isn't defined then the module name is used
     # It exists basically to allow hyphens in the command names
-    NAME = None
+    NAME: Optional[str] = None
     DESCRIPTION = ""  # Define this in the subclass
     _commands = None
 
@@ -309,66 +309,6 @@ class Command(object):
         )
 
 
-class TabularFieldsMixin(object):
-    """
-    A handy mixin to dump into classes that are expected to render tabular data.
-    It expects both that COLUMNS is defined by the subclass and that --field is
-    set in the add_arguments() method.
-    """
-
-    def _get_line_format(self):
-        """
-        Loop over the field arguments and generate a string that makes use of
-        Python's string format mini language.  We later use this string to
-        format the values for each row.
-        """
-        r = ""
-        for field in self.arguments.field:
-            if r:
-                r += " "
-            r += "{!s:" + u"{}{}".format(*self.COLUMNS[field]) + "}"
-        return r
-
-    def _get_header_names(self):
-        return [_.capitalize() for _ in self.arguments.field]
-
-    def _get_header(self):
-        """
-        Generates a header by using the line formatter and the list of field
-        arguments.
-        """
-        return self._get_line_format().format(*self._get_header_names())
-
-    def _get_horizontal_rule(self):
-        """
-        A bit of a hack: We get a formatted line for no other reason than to
-        determine the width of that line.  Then we use a regex to overwrite that
-        line with "=".
-        """
-        return re.sub(r".", "=", self._get_line_format().format(*self.arguments.field))
-
-    def _get_line_items(self, measurement):
-        raise NotImplementedError("This needs to be defined in the subclass.")
-
-    def _get_filter_display(self, filters):
-
-        if not filters:
-            return ""
-
-        r = colourise("\nFilters:\n", "white")
-        for k, v in filters.items():
-            if k not in ("search",):
-                v = str(v).capitalize()
-            r += colourise(
-                "  {}: {}\n".format(*self._get_filter_key_value_pair(k, v)), "cyan"
-            )
-
-        return r
-
-    def _get_filter_key_value_pair(self, k, v):
-        return k.capitalize().replace("__", " "), v
-
-
 class MetaDataMixin(object):
     @staticmethod
     def _prettify_boolean(boolean):
